// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.testing;

import com.goide.utils.GoSharedConstants;
import com.goide.utils.GoStringUtil;
import com.google.gson.annotations.SerializedName;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

class GoTestEvent {
  @Nullable
  @SerializedName("Action")
  public String action;

  @Nullable
  @SerializedName("Package")
  public String importPath;

  @Nullable
  @SerializedName("Test")
  public String testName;

  @SerializedName("Elapsed")
  public double elapsed;

  @Nullable
  @SerializedName("Output")
  public String output;

  @Nullable
  public GoTestJsonEventsProcessor.Action getAction() {
    if ("run".equals(action)) return GoTestJsonEventsProcessor.Action.RUN;
    if ("fail".equals(action)) return GoTestJsonEventsProcessor.Action.FAIL;
    if ("pass".equals(action)) return GoTestJsonEventsProcessor.Action.PASS;
    if ("skip".equals(action)) return GoTestJsonEventsProcessor.Action.SKIP;
    if ("output".equals(action)) return GoTestJsonEventsProcessor.Action.OUTPUT;
    if ("cont".equals(action)) return GoTestJsonEventsProcessor.Action.CONT;
    if ("pause".equals(action)) return GoTestJsonEventsProcessor.Action.PAUSE;
    return null;
  }

  @Nullable
  public String getNodeId() {
    return isTest() ? GoStringUtil.notNullize(importPath) + "#" + testName : importPath;
  }

  @Nullable
  public String getParentNodeId(@Nullable String parentTestName) {
    if (isTest()) {
      if (GoStringUtil.isNotEmpty(parentTestName)) {
        return GoStringUtil.notNullize(importPath) + "#" + parentTestName;
      }
      return importPath;
    }
    return null;
  }

  public boolean isTest() {
    return testName != null;
  }

  @NotNull
  public String getPackageLocationString(@NotNull String defaultImportPath) {
    return GoSharedConstants.TEST_PROTOCOL + "://" + GoStringUtil.notNullize(importPath, defaultImportPath) + "#";
  }

  @NotNull
  public String getTestLocationString(@NotNull String defaultImportPath) {
    return getPackageLocationString(defaultImportPath) + GoStringUtil.notNullize(testName);
  }

  @Nullable
  public String getName() {
    return isTest() ? testName : importPath;
  }

  @NotNull
  public String getElapsedTime() {
    return String.valueOf((int)(elapsed * 1000));
  }

  public boolean isPackage() {
    return testName == null && importPath != null;
  }
}
